#!/usr/bin/env python3

# Copyright 2020 Canonical Ltd.
# Licensed under the GPLv3, see LICENCE file for details.

import logging
from ops.charm import CharmBase
from ops.main import main
from ops.model import ActiveStatus, MaintenanceStatus
from pprint import pformat
from yaml import safe_load

logger = logging.getLogger()

REQUIRED_SETTINGS = ['bind_image_path']


class BindK8sCharm(CharmBase):
    def __init__(self, *args):
        """Initialise our class, we only care about 'start' and 'config-changed' hooks."""
        super().__init__(*args)
        self.framework.observe(self.on.start, self.on_config_changed)
        self.framework.observe(self.on.config_changed, self.on_config_changed)

    def _check_for_config_problems(self):
        """Return a string describing any configuration problems (or an empty string if none)."""
        problems = ''

        missing = self._missing_charm_settings()
        if missing:
            problems = 'required setting(s) empty: {}'.format(', '.join(sorted(missing)))

        return problems

    def _missing_charm_settings(self):
        """Return a list of missing configuration settings (or an empty list if none)."""
        config = self.model.config

        missing = {setting for setting in REQUIRED_SETTINGS if not config[setting]}

        if config['bind_image_username'] and not config['bind_image_password']:
            missing.add('bind_image_password')

        return missing

    def on_config_changed(self, event):
        """Check that we're leader, and if so, set up the pod."""
        if self.model.unit.is_leader():
            # Only the leader can set_spec().
            resources = self.make_pod_resources()
            spec = self.make_pod_spec()
            spec.update(resources)

            msg = "Configuring pod"
            logger.info(msg)
            self.model.unit.status = MaintenanceStatus(msg)
            self.model.pod.set_spec(spec)

            msg = "Pod configured"
            logger.info(msg)
            self.model.unit.status = ActiveStatus(msg)
        else:
            logger.info("Spec changes ignored by non-leader")
            self.model.unit.status = ActiveStatus()

    def make_pod_resources(self):
        """Compile and return our pod resources (e.g. ingresses)."""
        # LP#1889746: We need to define a manual ingress here to work around LP#1889703.
        resources = {}  # TODO
        logger.info("This is the Kubernetes Pod resources <<EOM\n{}\nEOM".format(pformat(resources)))
        return resources

    def generate_pod_config(self, secured=True):
        """Kubernetes pod config generator.

        generate_pod_config generates Kubernetes deployment config.
        If the secured keyword is set then it will return a sanitised copy
        without exposing secrets.
        """
        config = self.model.config
        pod_config = {}
        if config["container_config"].strip():
            pod_config = safe_load(config["container_config"])

        if config["custom_config_repo"].strip():
            pod_config["CUSTOM_CONFIG_REPO"] = config["custom_config_repo"]

        if config["https_proxy"].strip():
            pod_config["http_proxy"] = config["https_proxy"]
            pod_config["https_proxy"] = config["https_proxy"]

        if secured:
            return pod_config

        if config["container_secrets"].strip():
            container_secrets = safe_load(config["container_secrets"])
        else:
            container_secrets = {}

        pod_config.update(container_secrets)
        return pod_config

    def make_pod_spec(self):
        """Set up and return our full pod spec here."""
        config = self.model.config
        full_pod_config = self.generate_pod_config(secured=False)
        secure_pod_config = self.generate_pod_config(secured=True)

        ports = [
            {"name": "domain-tcp", "containerPort": 53, "protocol": "TCP"},
            {"name": "domain-udp", "containerPort": 53, "protocol": "UDP"},
        ]

        spec = {
            "version": 2,
            "containers": [
                {
                    "name": self.app.name,
                    "imageDetails": {"imagePath": config["bind_image_path"]},
                    "ports": ports,
                    "config": secure_pod_config,
                    "kubernetes": {"readinessProbe": {"exec": {"command": ["/usr/local/bin/dns-check.sh"]}}},
                }
            ],
        }

        logger.info("This is the Kubernetes Pod spec config (sans secrets) <<EOM\n{}\nEOM".format(pformat(spec)))

        if config.get("bind_image_username") and config.get("bind_image_password"):
            spec.get("containers")[0].get("imageDetails")["username"] = config["bind_image_username"]
            spec.get("containers")[0].get("imageDetails")["password"] = config["bind_image_password"]

        secure_pod_config.update(full_pod_config)

        return spec


if __name__ == "__main__":  # pragma: no cover
    main(BindK8sCharm)
